<?php

/**
 * @file
 * The mode module's administration functions.
 *
 * @author Karthik Kumar / Zen [ http://drupal.org/user/21209 ].
 */

/**
 * Menu callback: Mode module settings form.
 */
function mode_list() {
  $header = array(t('Name'), t('Status'), array('data' => t('Operations'), 'colspan' => 3));
  $rows = array();
  $modes = mode_mode_load();

  foreach ($modes as $mode) {
    $rows[] = array(
      check_plain($mode->name),
      $mode->active ? t('Enabled') : t('Disabled'),
      l(t('Edit'), 'admin/config/development/mode/edit/' . $mode->mid),
      l(t('Delete'), 'admin/config/development/mode/delete/' . $mode->mid),
      $mode->active ? l(t('Reapply'), 'admin/config/development/mode/switch/' . $mode->mid) : l(t('Apply'), 'admin/config/development/mode/switch/' . $mode->mid)
    );
  }

  return theme('table' , array('header' => $header, 'rows' => $rows, 'empty' => t('No modes have been created. Click on the !add-link tab to do so.', array('!add-link' => l(t('Add'), 'admin/config/development/mode/add')))));
}

/**
 * Menu callback: Mode settings form.
 */
function mode_settings_form($form, &$form_state) {
  $mode_config = _mode_variable_get();

  $form['mode_config'] = array('#tree' => TRUE);
  $form['mode_config']['banner'] = array(
    '#type' => 'checkbox',
    '#title' => t('Display banner message'),
    '#description' => t('If enabled, the banner message associated with a mode will be displayed on all pages as a status message.'),
    '#default_value' => $mode_config['banner']
  );
  $form['mode_config']['permissions_page'] = array(
    '#type' => 'checkbox',
    '#title' => t('Disable permissions page'),
    '#description' => t('If enabled, the permissions page will be disabled along with a message stating that the mode settings page should be used instead.'),
    '#default_value' => $mode_config['permissions_page']
  );

  return system_settings_form($form);
}

/**
 * Menu callback: Mode add / edit form.
 */
function mode_edit_form($form, &$form_state, $mode = NULL) {
  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Name'),
    '#description' => t('Give this mode a unique name for identification purposes.'),
    '#required' => TRUE,
  );
  $form['message'] = array(
    '#type' => 'textarea',
    '#title' => t('Banner message'),
    '#description' => t('Status message to display on all pages when this mode is enabled. HTML can be used. A toggle switch is also available on the settings page to control display.'),
    '#rows' => 2,
  );

  if ($mode) {
    drupal_set_title(t('Mode edit: !name', array('!name' => $mode->name)));
    $form['name']['#default_value'] = $mode->name;
    $form['message']['#default_value'] = $mode->message;
    $form['mid'] = array('#type' => 'value', '#value' => $mode->mid);
    $form['submit'] = array('#type' => 'submit', '#value' =>  t('Update mode'));
  }
  else {
    $form['submit'] = array('#type' => 'submit', '#value' => t('Add mode'));
  }

  return $form;
}

/**
 * Process mode edit form submissions.
 */
function mode_edit_form_submit($form, &$form_state) {
  $form_values = $form_state['values'];

  $mode = array('name' => trim($form_values['name']), 'message' => trim($form_values['message']));
  $update = array();
  if (isset($form_values['mid'])) {
    $mode['mid'] = $form_values['mid'];
    $update[] = 'mid';
  }
  $status = mode_mode_add($mode, $update);

  $form_state['redirect'] = 'admin/config/development/mode';
}

/**
 * Menu callback: Delete a mode.
 */
function mode_delete_form($form, &$form_state, $mode) {
  $form['mode'] = array('#type' => 'value', '#value' => $mode);
  return confirm_form($form,
    t('Are you sure you want to delete the mode %name?', array('%name' => $mode->name)),
    'admin/config/development/mode',
    t('This will delete the mode permanently. This action cannot be undone.'),
    t('Delete'),
    t('Cancel')
  );
}

/**
 * Process mode delete form submissions.
 */
function mode_delete_form_submit($form, &$form_state) {
  mode_mode_delete($form_state['values']['mode']);
  $form_state['redirect'] = 'admin/config/development/mode';
}

/**
 * Confirmation form for mode switching.
 */
function mode_mode_switch_form($form, &$form_state, $mode) {
  if (isset($mode->mid)) {
    $form['mode'] = array('#type' => 'value', '#value' => $mode);
    return confirm_form($form,
      t('Are you sure you want to switch to mode %name?', array('%name' => $mode->name)),
      'admin/config/development/mode',
      $mode->message,
      t('Switch'),
      t('Cancel')
    );
  }
  else {
    drupal_set_message(t('No such mode is available.'));
    drupal_goto('admin/config/development/mode');
  }
}

/**
 * Process mode delete form submissions.
 */
function mode_mode_switch_form_submit($form, &$form_state) {
  $mode = mode_mode_switch($form_state['values']['mode']->mid);

  if ($mode) {
    drupal_set_message(t('Mode switched to %mode.', array('%mode' => $mode->name)));
  }

  $form_state['redirect'] = 'admin/config/development/mode';
}
