<?php
/**
 * @file
 * Administer Mode permissions
 */

/**
 * Menu callback: Mode add/edit form.
 */
function mode_modules_edit_form($form, &$form_state, $mode) {
  $form['mode'] = array('#type' => 'value', '#value' => $mode);
  // Get current list of modules and sort by low level module name.
  $files = _system_rebuild_module_data();
  ksort($files);
  $form['list'] = array(
    '#tree' => TRUE,
    '#theme' => 'mode_modules_edit_table',
  );
  $state = mode_data_load($mode->mid, 'modules');
  foreach ($files as $name => $module) {
    // Exclude system and hidden ones.
    if (!empty($module->info['required']) || !empty($module->info['hidden'])) continue;
    $form['list'][$name]['name'] = array(
      '#type' => 'markup',
      '#markup' => $module->info['name']
    );
    $form['list'][$name]['description'] = array(
      '#type' => 'markup',
      '#markup' => $module->info['description']
    );
    if (isset($state[$name])) {
      $default = $state[$name] ? 1 : 2;
    }
    else {
      $default = 0;
    }
    $form['list'][$name]['enable'] = array(
      '#type' => 'radio',
      '#return_value' => 1,
      '#default_value' => $default,
      '#parents' => array('list', $name)
    );
    $form['list'][$name]['disable'] = array(
      '#type' => 'radio',
      '#return_value' => 2,
      '#default_value' => $default,
      '#parents' => array('list', $name)
    );
    $form['list'][$name]['none'] = array(
      '#type' => 'radio',
      '#return_value' => 0,
      '#default_value' => $default,
      '#parents' => array('list', $name)
    );
  }
  $form['submit'] = array(
    '#value' => t('Save'),
    '#type' => 'submit',
  );
  return $form;
}

/**
 * Process mode edit form submissions.
 */
function mode_modules_edit_form_submit($form, &$form_state) {
  $mode = $form_state['values']['mode'];
  $list = array_filter($form_state['values']['list']);
  $modules = array();
  foreach ($list as $name => $state) {
    $modules[$name] = $state == 1;
  }
  $status = mode_data_save($mode->mid, 'modules', $modules);

  //$form_state['redirect'] = 'admin/config/development/mode/';
}

/**
 * Theme function
 */
function theme_mode_modules_edit_table($variables) {
  $element = $variables['element'];
  $header = array(t('Module'), t('Name'), t('Description'), t('Enable'), t('Disable'), t('Keep'));
  $rows = array();
  foreach (element_children($element) as $name) {
    $rows[] = array(
      array('data' => $name, 'header' => TRUE),
      drupal_render($element[$name]['name']),
      drupal_render($element[$name]['description']),
      drupal_render($element[$name]['enable']),
      drupal_render($element[$name]['disable']),
      drupal_render($element[$name]['none']),
    );
  }
  $output = theme('table', array('header' => $header, 'rows' => $rows));
  return $output;
}
