<?php
/**
 * @file
 * Administer Mode permissions
 */

/**
 * Menu callback: Mode variables add/edit form.
 */
function mode_variables_edit_form($form, &$form_state, $mode) {
  module_load_include('form.inc', 'variable');
  $form['mode'] = array('#type' => 'value', '#value' => $mode);
  // Select variables from list.
  $form['select'] = array(
    '#title' => t('Select variables'),
    '#type' => 'fieldset',
  );
  $options = _mode_variables_options_list();
  $options[''] = t('<select variable to add>');
  $form['select']['variable_name'] = array(
    '#type' => 'select',
    '#options' => $options,
    '#default_value' => '',
  );
  $form['select']['add'] = array(
    '#name' => 'variable-add',
    '#type' => 'submit',
    '#value' => t('Add'),
    '#submit' => array('mode_variables_edit_form_list_add'),
  );
  // Add warning message if changes made but not submitted.
  if (!empty($form_state['variable_changed'])) {
    $form['message']['#markup'] = '<div class="messages warning">' . t("There are changes that won't be saved until you click on <em>Save</em>.") . '</div>';
  }
  // Set variable values for selected variables.
  $mode_data = mode_data_load($mode->mid, 'variables');
  if (isset($form_state['variable_list'])) {
    $variable_list = $form_state['variable_list'];
  }
  else {
    $variable_list = array_keys($mode_data);
    $form_state['variable_list'] = $variable_list ? array_combine($variable_list, $variable_list) : array();
  }
  $form['variables'] = array(
    //'#title' => t('Set values'),
    //'#type' => 'fieldset',
    '#tree' => TRUE,
  );
  // Tree for remove button.
  $form['remove']['#tree'] = TRUE;
  foreach ($variable_list as $name) {
    $variable = variable_build($name);
    if (isset($mode_data[$name])) {
      $variable['value'] = $mode_data[$name];
    }
    $form['variables'][$name] = variable_form_element($variable);
    $form['remove'][$name] = array(
      '#type' => 'submit',
      '#value' => t('Remove'),
      '#submit' => array('mode_variables_edit_form_list_remove'),
    );
  }
  $form['submit'] = array(
    '#value' => t('Save'),
    '#type' => 'submit',
  );
  return $form;
}

/**
 * Form button submit callback: Add variable to form.
 */
function mode_variables_edit_form_list_add($form, &$form_state) {
  if ($variable_name = $form_state['values']['variable_name']) {
    foreach (variable_children($variable_name) as $name) {
      $form_state['variable_list'][$name] = $name;
    }
    $form_state['variable_changed'] = TRUE;
    $form_state['rebuild'] = TRUE;
  }
}
/**
 * Form button submit callback: Remove variable from form.
 */
function mode_variables_edit_form_list_remove($form, &$form_state) {
  $button = $form_state['clicked_button'];
  $name = array_pop($button['#parents']);
  unset($form_state['variable_list'][$name]);
  $form_state['variable_changed'] = TRUE;
  $form_state['rebuild'] = TRUE;
}

/**
 * Process mode edit form submissions.
 */
function mode_variables_edit_form_submit($form, &$form_state) {
  $mode = $form_state['values']['mode'];
  $variables = isset($form_state['values']['variables']) ? $form_state['values']['variables'] : array();
  $status = mode_data_save($mode->mid, 'variables', $variables);
  drupal_set_message(t('The variable values have been saved'));
  //$form_state['redirect'] = 'admin/config/development/mode/';
}

/**
 * Helper functon; Build variables options list.
 */
function _mode_variables_options_list() {
  $variables = &drupal_static(__FUNCTION__, array());
  $module_info = system_get_info('module');
  foreach (variable_get_info() as $name => $variable) {
    if (!isset($variables[$module_info[$variable['module']]['name']])) {
      $variables[$module_info[$variable['module']]['name']] = array();
    }
    $variables[$module_info[$variable['module']]['name']][$name] = $variable['title'];
    if (isset($variable['multiple'])) {
      $variable = variable_build($variable);
      foreach ($variable['children'] as $name => $child) {
        $variables[$module_info[$variable['module']]['name']][$name] = "-- {$variable['title']} [{$child['title']}]";
      }
    }
  }
  ksort($variables);
  foreach ($variables as &$group) {
    ksort($group);
  }
  return $variables;
}

/**
 * Theme variables form
 */
function theme_mode_variables_edit_form($variables) {
  $form = $variables['form'];
  $output = drupal_render($form['select']);
  if (isset($form['message'])) {
    $output .= drupal_render($form['message']);
  }
  foreach (element_children($form['variables']) as $name) {
    // Take out description.
    if (isset($form['variables'][$name]['#description'])) {
      $description = $form['variables'][$name]['#description'];
      unset($form['variables'][$name]['#description']);
    }
    else {
      $description = '';
    }
    $rows[] = array(
      array('data' => $name, 'header' => TRUE),
      drupal_render($form['variables'][$name]),
      drupal_render($form['remove'][$name])
    );
  }
  if (!empty($rows)) {
    $output .= theme('table', array('rows' => $rows));
  }

  $output .= drupal_render_children($form);
  return $output;
}
